;+
; NAME:
;  matchup
; PURPOSE:
;  Read a batch of FITS files and create a list of common object exposures.
; DESCRIPTION:
;
; CATEGORY:
;  Astrometry
; CALLING SEQUENCE:
;  matchup,root
; INPUTS:
;  root - String, root of file name for the FITS files.
; OPTIONAL INPUT PARAMETERS:
;
; KEYWORD INPUT PARAMETERS:
;  OUTFILE - Name of output match file, default = root+'.match'
;  PATH - Path of where to find the source images, default=current directory.
; OUTPUTS:
;
; KEYWORD OUTPUT PARAMETERS:
;
; COMMON BLOCKS:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; MODIFICATION HISTORY:
; 98/06/23, Written by Marc W. Buie, Lowell Observatory
;
;-
pro matchup,root,OUTFILE=outfile,PATH=path

   if badpar(root,7,0,CALLER='MATCHUP: (root) ') then return
   if badpar(outfile,[0,7],0,CALLER='MATCHUP: (OUTFILE) ', $
                default=root+'.match') then return
   if badpar(path,[0,7],0,CALLER='MATCHUP: (PATH) ',default='./') then return

   path=addslash(path)

   if exists(outfile) then begin
      print,'Warning: matchup file ',outfile,' already exists.'
      ans=''
      read,ans,prompt='Do you wish to overwrite? ',ans
      if strlowcase(strmid(ans,0,1)) ne 'y' then return
   endif

   blanks='                '

   ; Get the list of files.
   spawn,'cd '+path+' ; ls '+root+'.???',file
   nfiles=n_elements(file)
   object=strarr(nfiles)
   for i=0,nfiles-1 do begin
      hdr=headfits(path+file[i])
      object[i]=strupcase(strcompress(sxpar(hdr,'OBJECT'),/remove_all))
   endfor

   ; Collate the list by unique object name.
   objlist=object[uniq(object,sort(object))]
   nobj=n_elements(objlist)
   ffn=strarr(nobj)
   for i=0,nobj-1 do begin
      z=where(objlist[i] eq object,count)
      ffn[i]=file[z[0]]
   endfor
   idx=sort(ffn)
   objlist=objlist[idx]

   ; Save results to output file.
   openw,lun,outfile,/get_lun
   for i=0,nobj-1 do begin
      z=where(objlist[i] eq object,count)
      printf,lun,objlist[i]+blanks,file[z],format='(a16,10(1x,a))'
      if count ne 3 then print,'Warning ',objlist[i],' has ',count,' entries.'
   endfor
   free_lun,lun

end
