;+
; NAME:
;  raparse
; PURPOSE:
;  Convert Right Ascension (RA) string to radians.
; DESCRIPTION:
;
; CATEGORY:
;  Astronomy
; CALLING SEQUENCE:
;  ra=raparse(str)
; INPUTS:
;  str - String (or array) to parse as a right ascension
; OPTIONAL INPUT PARAMETERS:
;
; KEYWORD INPUT PARAMETERS:
;
; OUTPUTS:
;  return value is scalar or vector value of RA in radians
; KEYWORD OUTPUT PARAMETERS:
;
; COMMON BLOCKS:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURE:
;
; MODIFICATION HISTORY:
;  1997/06/04, Written by Marc W. Buie, Lowell Observatory
;-
FUNCTION raparse,str

   ra=dblarr(n_elements(str))

   FOR i=0,n_elements(str)-1 DO BEGIN
      ; Make copy of string
      wstr = str[i]

      ; convert separator tokens to blank
      wstr = repchar(wstr,':',' ')
      wstr = repchar(wstr,',',' ')

      ; split into fields by blanks
      raf=double(str_sep(wstr,' '))
      IF n_elements(raf) eq 1 THEN BEGIN
         raf = [raf,0.,0.]
      ENDIF ELSE IF n_elements(raf) EQ 2 THEN BEGIN
         raf = [raf[*],0.]
      ENDIF
      hmstorad,raf[0],raf[1],raf[2],ra0
      ra[i] = ra0
   ENDFOR

   IF n_elements(ra) eq 1 THEN return,ra[0] ELSE return,ra

END
